@@ -1,6 +1,6 @@
 """Main entry point for deepfabric CLI."""
 
-from .cli import cli
+from deepfabric.cli import cli
 
 if __name__ == "__main__":
     cli()
