@@ -12,6 +12,11 @@ from click.testing import CliRunner
 from deepfabric.cli import cli
 
 
+async def _async_iter(items):
+    for item in items:
+        yield item
+
+
 @pytest.fixture
 def cli_runner():
     """Create a CLI runner for testing."""
@@ -128,7 +133,7 @@ def test_generate_help(cli_runner):
     assert "--sys-msg" in result.output
 
 
-@patch("deepfabric.cli.create_topic_generator")
+@patch("deepfabric.topic_manager.create_topic_generator")
 @patch("deepfabric.cli.DataSetGenerator")
 def test_generate_command_basic(
     mock_data_engine, mock_create_topic_generator, cli_runner, sample_config_file
@@ -139,7 +144,7 @@ def test_generate_command_basic(
     from deepfabric.tree import Tree  # noqa: PLC0415
 
     mock_tree_instance = Mock(spec=Tree)
-    mock_tree_instance.build.return_value = iter(
+    mock_tree_instance.build_async.return_value = _async_iter(
         [{"event": "build_complete", "total_paths": 9, "failed_generations": 0}]
     )  # Make build return proper event
     mock_tree_instance.tree_paths = [
@@ -152,7 +157,7 @@ def test_generate_command_basic(
 
     mock_create_topic_generator.return_value = mock_tree_instance
     mock_data_engine.return_value = mock_engine_instance
-    mock_engine_instance.create_data_with_events.return_value = iter(
+    mock_engine_instance.create_data_with_events_async.return_value = _async_iter(
         [
             {"event": "generation_complete", "total_samples": 5, "failed_samples": 0},
             mock_dataset,  # Yield the dataset as final result
@@ -167,14 +172,14 @@ def test_generate_command_basic(
 
     # Verify mocks were called correctly
     mock_create_topic_generator.assert_called_once()
-    mock_tree_instance.build.assert_called_once()
+    mock_tree_instance.build_async.assert_called_once()
     mock_tree_instance.save.assert_called_once()
     mock_data_engine.assert_called_once()
-    mock_engine_instance.create_data_with_events.assert_called_once()
+    mock_engine_instance.create_data_with_events_async.assert_called_once()
     mock_dataset.save.assert_called_once()
 
 
-@patch("deepfabric.cli.create_topic_generator")
+@patch("deepfabric.topic_manager.create_topic_generator")
 @patch("deepfabric.cli.DataSetGenerator")
 def test_generate_command_with_sys_msg_override(
     mock_data_engine, mock_create_topic_generator, cli_runner, sample_config_file
@@ -185,7 +190,7 @@ def test_generate_command_with_sys_msg_override(
     from deepfabric.tree import Tree  # noqa: PLC0415
 
     mock_tree_instance = Mock(spec=Tree)
-    mock_tree_instance.build.return_value = iter(
+    mock_tree_instance.build_async.return_value = _async_iter(
         [{"event": "build_complete", "total_paths": 9, "failed_generations": 0}]
     )
     mock_tree_instance.tree_paths = [["root", "child1"], ["root", "child2"]]
@@ -195,7 +200,7 @@ def test_generate_command_with_sys_msg_override(
 
     mock_create_topic_generator.return_value = mock_tree_instance
     mock_data_engine.return_value = mock_engine_instance
-    mock_engine_instance.create_data_with_events.return_value = iter(
+    mock_engine_instance.create_data_with_events_async.return_value = _async_iter(
         [{"event": "generation_complete", "total_samples": 5, "failed_samples": 0}, mock_dataset]
     )
 
@@ -214,11 +219,11 @@ def test_generate_command_with_sys_msg_override(
     assert result.exit_code == 0
 
     # Verify create_data was called with sys_msg=False
-    args, kwargs = mock_engine_instance.create_data_with_events.call_args
+    args, kwargs = mock_engine_instance.create_data_with_events_async.call_args
     assert kwargs["sys_msg"] is False
 
 
-@patch("deepfabric.cli.create_topic_generator")
+@patch("deepfabric.topic_manager.create_topic_generator")
 @patch("deepfabric.cli.DataSetGenerator")
 def test_generate_command_default_sys_msg(
     mock_data_engine, mock_create_topic_generator, cli_runner, sample_config_file_no_sys_msg
@@ -229,7 +234,7 @@ def test_generate_command_default_sys_msg(
     from deepfabric.tree import Tree  # noqa: PLC0415
 
     mock_tree_instance = Mock(spec=Tree)
-    mock_tree_instance.build.return_value = iter(
+    mock_tree_instance.build_async.return_value = _async_iter(
         [{"event": "build_complete", "total_paths": 9, "failed_generations": 0}]
     )
     mock_tree_instance.tree_paths = [["root", "child1"], ["root", "child2"]]
@@ -239,7 +244,7 @@ def test_generate_command_default_sys_msg(
 
     mock_create_topic_generator.return_value = mock_tree_instance
     mock_data_engine.return_value = mock_engine_instance
-    mock_engine_instance.create_data_with_events.return_value = iter(
+    mock_engine_instance.create_data_with_events_async.return_value = _async_iter(
         [{"event": "generation_complete", "total_samples": 5, "failed_samples": 0}, mock_dataset]
     )
 
@@ -250,11 +255,11 @@ def test_generate_command_default_sys_msg(
     assert result.exit_code == 0
 
     # Verify create_data was called with default sys_msg (should be None to use engine default)
-    args, kwargs = mock_engine_instance.create_data_with_events.call_args
+    args, kwargs = mock_engine_instance.create_data_with_events_async.call_args
     assert "sys_msg" not in kwargs or kwargs["sys_msg"] is None
 
 
-@patch("deepfabric.cli.create_topic_generator")
+@patch("deepfabric.topic_manager.create_topic_generator")
 @patch("deepfabric.cli.DataSetGenerator")
 def test_generate_command_with_overrides(
     mock_data_engine, mock_create_topic_generator, cli_runner, sample_config_file
@@ -265,7 +270,7 @@ def test_generate_command_with_overrides(
     from deepfabric.tree import Tree  # noqa: PLC0415
 
     mock_tree_instance = Mock(spec=Tree)
-    mock_tree_instance.build.return_value = iter(
+    mock_tree_instance.build_async.return_value = _async_iter(
         [{"event": "build_complete", "total_paths": 9, "failed_generations": 0}]
     )
     mock_tree_instance.tree_paths = [["root", "child1"], ["root", "child2"]]
@@ -275,7 +280,7 @@ def test_generate_command_with_overrides(
 
     mock_create_topic_generator.return_value = mock_tree_instance
     mock_data_engine.return_value = mock_engine_instance
-    mock_engine_instance.create_data_with_events.return_value = iter(
+    mock_engine_instance.create_data_with_events_async.return_value = _async_iter(
         [{"event": "generation_complete", "total_samples": 5, "failed_samples": 0}, mock_dataset]
     )
 
@@ -317,15 +322,15 @@ def test_generate_command_with_overrides(
     mock_tree_instance.save.assert_called_once_with("override_tree.jsonl")
     mock_dataset.save.assert_called_once_with("override_dataset.jsonl")
 
-    args, kwargs = mock_engine_instance.create_data_with_events.call_args
+    args, kwargs = mock_engine_instance.create_data_with_events_async.call_args
     assert kwargs["num_steps"] == 10  # noqa: PLR2004
     assert kwargs["batch_size"] == 2  # noqa: PLR2004
     assert kwargs["model_name"] == "model"  # Updated expectation based on current CLI behavior
     assert kwargs["sys_msg"] is False
 
 
-@patch("deepfabric.cli.read_topic_tree_from_jsonl")
-@patch("deepfabric.cli.Tree")
+@patch("deepfabric.topic_manager.read_topic_tree_from_jsonl")
+@patch("deepfabric.topic_manager.Tree")
 @patch("deepfabric.cli.DataSetGenerator")
 def test_generate_command_with_jsonl(
     mock_data_engine,
@@ -346,7 +351,7 @@ def test_generate_command_with_jsonl(
     mock_data_engine.return_value = mock_engine_instance
     mock_dataset = Mock(spec=Dataset)
     mock_engine_instance.dataset = mock_dataset
-    mock_engine_instance.create_data_with_events.return_value = iter(
+    mock_engine_instance.create_data_with_events_async.return_value = _async_iter(
         [{"event": "generation_complete", "total_samples": 5, "failed_samples": 0}, mock_dataset]
     )
     # Create a temporary JSONL file
@@ -405,7 +410,7 @@ def test_generate_command_invalid_yaml(cli_runner):
             os.unlink(temp_path)
 
 
-@patch("deepfabric.cli.create_topic_generator")
+@patch("deepfabric.topic_manager.create_topic_generator")
 @patch("deepfabric.cli.DataSetGenerator")
 def test_generate_command_error_handling(
     _mock_data_engine,
@@ -422,7 +427,7 @@ def test_generate_command_error_handling(
 
     # Verify command failed with error
     assert result.exit_code != 0
-    assert "Error" in result.output
+    assert "error" in result.output.lower()  # Case-insensitive check
     assert "Test error" in result.output
 
 
@@ -475,7 +480,7 @@ def test_upload_command(mock_uploader, cli_runner):
             os.unlink(temp_path)
 
 
-@patch("deepfabric.cli.Graph.from_json")
+@patch("deepfabric.topic_manager.Graph.from_json")
 def test_visualize_command(mock_from_json, cli_runner):
     """Test visualize command."""
     # Create a temporary graph JSON file
@@ -503,3 +508,143 @@ def test_visualize_command(mock_from_json, cli_runner):
         # Cleanup
         if os.path.exists(temp_path):
             os.unlink(temp_path)
+
+
+@patch("deepfabric.topic_manager.create_topic_generator")
+def test_topic_only_flag_tree(mock_create_topic_generator, cli_runner, sample_config_file):
+    """Test --topic-only flag with tree mode."""
+    from deepfabric.tree import Tree  # noqa: PLC0415
+
+    # Setup mock tree
+    mock_tree_instance = Mock(spec=Tree)
+    mock_tree_instance.build_async.return_value = _async_iter(
+        [{"event": "build_complete", "total_paths": 9, "failed_generations": 0}]
+    )
+    mock_tree_instance.tree_paths = [["root", "child1"], ["root", "child2"]]
+    mock_create_topic_generator.return_value = mock_tree_instance
+
+    # Run command with --topic-only
+    result = cli_runner.invoke(cli, ["generate", sample_config_file, "--topic-only"])
+
+    # Verify command executed successfully
+    assert result.exit_code == 0
+
+    # Verify tree was built and saved
+    mock_create_topic_generator.assert_called_once()
+    mock_tree_instance.build_async.assert_called_once()
+    mock_tree_instance.save.assert_called_once_with("test_tree.jsonl")
+
+    # Verify success message about topic save
+    assert "Topic tree saved to" in result.output
+
+
+@patch("deepfabric.topic_manager.create_topic_generator")
+def test_topic_only_flag_graph(mock_create_topic_generator, cli_runner):
+    """Test --topic-only flag with graph mode."""
+    from deepfabric.graph import Graph  # noqa: PLC0415
+
+    # Setup mock graph
+    mock_graph_instance = Mock(spec=Graph)
+    mock_graph_instance.build_async.return_value = _async_iter(
+        [{"event": "build_complete", "failed_generations": 0}]
+    )
+    mock_create_topic_generator.return_value = mock_graph_instance
+
+    # Create sample graph config
+    graph_yaml = """
+dataset_system_prompt: "Test system prompt"
+topic_graph:
+  args:
+    topic_prompt: "Test root prompt"
+    topic_system_prompt: "Test system prompt"
+    degree: 3
+    depth: 2
+    temperature: 0.7
+    provider: "test"
+    model: "model"
+  save_as: "test_graph.json"
+data_engine:
+  args:
+    instructions: "Test instructions"
+    generation_system_prompt: "Test system prompt"
+    provider: "test"
+    model: "model"
+    temperature: 0.9
+    max_retries: 2
+dataset:
+  creation:
+    num_steps: 5
+    batch_size: 1
+    provider: "test"
+    model: "model"
+    sys_msg: true
+  save_as: "test_dataset.jsonl"
+"""
+    with tempfile.NamedTemporaryFile(mode="w", suffix=".yaml", delete=False) as f:
+        f.write(graph_yaml)
+        temp_config = f.name
+
+    try:
+        # Run command with --topic-only and --mode graph
+        result = cli_runner.invoke(
+            cli, ["generate", temp_config, "--mode", "graph", "--topic-only"]
+        )
+
+        # Verify command executed successfully
+        assert result.exit_code == 0
+
+        # Verify graph was built and saved
+        mock_create_topic_generator.assert_called_once()
+        mock_graph_instance.build_async.assert_called_once()
+        mock_graph_instance.save.assert_called_once_with("test_graph.json")
+
+        # Verify success message about topic save
+        assert "Topic graph saved to" in result.output
+
+    finally:
+        if os.path.exists(temp_config):
+            os.unlink(temp_config)
+
+
+def test_topic_only_with_load_tree_fails(cli_runner, sample_config_file):
+    """Test that --topic-only fails when used with --load-tree."""
+    # Create a temporary JSONL file
+    with tempfile.NamedTemporaryFile(mode="w", suffix=".jsonl", delete=False) as f:
+        f.write('{"path": ["root", "child"]}\n')
+        temp_jsonl_path = f.name
+
+    try:
+        result = cli_runner.invoke(
+            cli,
+            ["generate", sample_config_file, "--topic-only", "--load-tree", temp_jsonl_path],
+        )
+
+        # Should fail validation
+        assert result.exit_code != 0
+        assert "--topic-only cannot be used with --load-tree or --load-graph" in result.output
+
+    finally:
+        if os.path.exists(temp_jsonl_path):
+            os.unlink(temp_jsonl_path)
+
+
+def test_topic_only_with_load_graph_fails(cli_runner, sample_config_file):
+    """Test that --topic-only fails when used with --load-graph."""
+    # Create a temporary JSON file
+    with tempfile.NamedTemporaryFile(mode="w", suffix=".json", delete=False) as f:
+        f.write('{"nodes": {}, "edges": []}')
+        temp_json_path = f.name
+
+    try:
+        result = cli_runner.invoke(
+            cli,
+            ["generate", sample_config_file, "--topic-only", "--load-graph", temp_json_path],
+        )
+
+        # Should fail validation
+        assert result.exit_code != 0
+        assert "--topic-only cannot be used with --load-tree or --load-graph" in result.output
+
+    finally:
+        if os.path.exists(temp_json_path):
+            os.unlink(temp_json_path)
