@@ -23,7 +23,18 @@ from deepfabric.formatters.builtin.chatml import ChatmlFormatter
 from deepfabric.formatters.builtin.grpo import GrpoFormatter
 from deepfabric.formatters.builtin.harmony import HarmonyFormatter
 from deepfabric.formatters.builtin.tool_calling import ToolCallingFormatter
+from deepfabric.formatters.builtin.trl_sft_tools import TRLSFTToolsFormatter
 from deepfabric.formatters.registry import FormatterRegistry
+from deepfabric.schemas import (
+    ChatMessage,
+    Conversation,
+    ReasoningTrace,
+    ToolContext,
+    ToolDefinition,
+    ToolExecution,
+    ToolParameter,
+    ToolRegistry,
+)
 
 FORMATTED_LENGTH = 3
 
@@ -590,17 +601,36 @@ class TestToolCallingFormatter:
 
     def test_tool_calling_formatter_basic(self):
         """Test basic tool calling formatter functionality."""
+
         formatter = ToolCallingFormatter()
 
-        # Simple agent reasoning sample
-        sample = {
-            "question": "What's the weather in London?",
-            "reasoning": "I need to check the current weather for London using the weather tool.",
-            "tool_used": "get_weather",
-            "tool_input": '{"location": "London, UK"}',
-            "tool_output": "18°C, cloudy with light rain",
-            "answer": "The weather in London is currently 18°C with cloudy skies and light rain.",
-        }
+        # Unified Conversation schema sample with reasoning and tool context
+        sample = Conversation(
+            messages=[
+                ChatMessage(role="user", content="What's the weather in London?"),
+                ChatMessage(
+                    role="assistant",
+                    content="The weather in London is currently 18°C with cloudy skies and light rain.",
+                ),
+            ],
+            reasoning=ReasoningTrace(
+                style="freetext",
+                content="I need to check the current weather for London using the weather tool. Check weather for London. Weather tool is appropriate for this query. Use London, UK as location parameter. Tool returned current weather conditions.",
+            ),
+            tool_context=ToolContext(
+                available_tools=[],
+                executions=[
+                    ToolExecution(
+                        function_name="get_weather",
+                        arguments='{"location": "London, UK"}',
+                        reasoning="Need current weather data for London",
+                        result="18°C, cloudy with light rain",
+                    )
+                ],
+            ),
+            question="What's the weather in London?",
+            final_answer="The weather in London is currently 18°C with cloudy skies and light rain.",
+        )
 
         result = formatter.format_with_metadata([sample])
         assert len(result.samples) == 1
@@ -618,7 +648,7 @@ class TestToolCallingFormatter:
         assert roles == ["user", "assistant", "tool", "assistant"]
 
         # Check user message
-        assert messages[0]["content"] == sample["question"]
+        assert messages[0]["content"] == sample.question
 
         # Check assistant message with thinking and tool call
         assistant_msg = messages[1]["content"]
@@ -626,99 +656,102 @@ class TestToolCallingFormatter:
         assert "</think>" in assistant_msg
         assert "<tool_call>" in assistant_msg
         assert "</tool_call>" in assistant_msg
-        assert sample["reasoning"] in assistant_msg
 
         # Check tool response
         tool_msg = messages[2]["content"]
         assert "<tool_response>" in tool_msg
         assert "</tool_response>" in tool_msg
-        assert sample["tool_output"] in tool_msg
+        assert "18°C, cloudy with light rain" in tool_msg
 
         # Check final answer
-        assert messages[3]["content"] == sample["answer"]
+        assert "18°C with cloudy skies and light rain" in messages[3]["content"]
 
     def test_tool_calling_formatter_rich_reasoning(self):
         """Test tool calling formatter with rich CoT reasoning."""
+
         formatter = ToolCallingFormatter()
 
-        # Rich agent reasoning sample
-        sample = {
-            "question": "Calculate the square root of 144",
-            "initial_analysis": "The user wants me to find the square root of 144, which is a mathematical calculation.",
-            "reasoning_steps": [
-                "I need to perform a mathematical operation (square root)",
-                "The calculator tool is the appropriate choice for this task",
-                "I'll use the sqrt operation with value 144",
+        # Unified Conversation with structured reasoning
+        reasoning_content = "The user wants me to find the square root of 144, which is a mathematical calculation. I need to perform a mathematical operation (square root). The calculator tool is the appropriate choice for this task. I'll use the sqrt operation with value 144."
+
+        sample = Conversation(
+            messages=[
+                ChatMessage(role="user", content="Calculate the square root of 144"),
+                ChatMessage(role="assistant", content="The square root of 144 is 12."),
             ],
-            "tool_selection_rationale": "The calculator tool is designed for mathematical operations and has a sqrt function.",
-            "parameter_reasoning": "I need to set operation to 'sqrt' and value to 144 for the calculation.",
-            "result_interpretation": "The tool returned 12, which is the correct square root of 144.",
-            "tool_used": "calculator",
-            "tool_input": '{"operation": "sqrt", "value": 144}',
-            "tool_output": "12",
-            "answer": "The square root of 144 is 12.",
-        }
+            reasoning=ReasoningTrace(style="freetext", content=reasoning_content),
+            tool_context=ToolContext(
+                available_tools=[],
+                executions=[
+                    ToolExecution(
+                        function_name="calculator",
+                        arguments='{"operation": "sqrt", "value": 144}',
+                        reasoning="Calculate square root of 144",
+                        result="12",
+                    )
+                ],
+            ),
+            question="Calculate the square root of 144",
+            final_answer="The square root of 144 is 12.",
+        )
 
         result = formatter.format_with_metadata([sample])
         formatted = result.samples[0]
         messages = formatted["messages"]
 
-        # Check that rich reasoning is included in thinking
+        # Check that reasoning is included in thinking
         assistant_msg = messages[1]["content"]
-        assert "Analysis:" in assistant_msg
-        assert sample["initial_analysis"] in assistant_msg
-        assert "Step-by-step reasoning:" in assistant_msg
-        assert "Tool selection:" in assistant_msg
-        assert sample["tool_selection_rationale"] in assistant_msg
-        assert "Parameters:" in assistant_msg
-        assert sample["parameter_reasoning"] in assistant_msg
-
-        # Verify all reasoning steps are included
-        for step in sample["reasoning_steps"]:
-            assert step in assistant_msg
+        assert "<think>" in assistant_msg
+        assert "mathematical calculation" in assistant_msg
+        assert "square root" in assistant_msg
 
     def test_tool_calling_formatter_with_tools_in_system(self):
         """Test tool calling formatter with tools included in system message."""
+
         config = {
             "include_tools_in_system": True,
             "system_prompt": "You are a helpful AI assistant with access to tools.",
         }
-        formatter = ToolCallingFormatter(config)
 
-        sample = {
-            "question": "What's 5 + 3?",
-            "reasoning": "I need to add two numbers.",
-            "tool_used": "calculator",
-            "tool_input": '{"operation": "add", "a": 5, "b": 3}',
-            "tool_output": "8",
-            "answer": "5 + 3 = 8",
-            "available_tools": [
-                {
-                    "name": "calculator",
-                    "description": "Perform mathematical calculations",
-                    "parameters": [
-                        {
-                            "name": "operation",
-                            "type": "str",
-                            "description": "Math operation",
-                            "required": True,
-                        },
-                        {
-                            "name": "a",
-                            "type": "int",
-                            "description": "First number",
-                            "required": True,
-                        },
-                        {
-                            "name": "b",
-                            "type": "int",
-                            "description": "Second number",
-                            "required": True,
-                        },
-                    ],
-                }
+        # Create tool registry with calculator tool
+        calculator_tool = ToolDefinition(
+            name="calculator",
+            description="Perform mathematical calculations",
+            parameters=[
+                ToolParameter(
+                    name="operation", type="str", description="Math operation", required=True
+                ),
+                ToolParameter(name="a", type="int", description="First number", required=True),
+                ToolParameter(name="b", type="int", description="Second number", required=True),
             ],
-        }
+            returns="The result of the mathematical operation",
+        )
+        tool_registry = ToolRegistry(tools=[calculator_tool])
+
+        formatter = ToolCallingFormatter(config, tool_registry=tool_registry)
+
+        sample = Conversation(
+            messages=[
+                ChatMessage(role="user", content="What's 5 + 3?"),
+                ChatMessage(role="assistant", content="5 + 3 = 8"),
+            ],
+            reasoning=ReasoningTrace(
+                style="freetext", content="I need to add two numbers. Add 5 and 3."
+            ),
+            tool_context=ToolContext(
+                available_tools=[calculator_tool],
+                executions=[
+                    ToolExecution(
+                        function_name="calculator",
+                        arguments='{"operation": "add", "a": 5, "b": 3}',
+                        reasoning="Add two numbers",
+                        result="8",
+                    )
+                ],
+            ),
+            question="What's 5 + 3?",
+            final_answer="5 + 3 = 8",
+        )
 
         result = formatter.format_with_metadata([sample])
         formatted = result.samples[0]
@@ -737,23 +770,35 @@ class TestToolCallingFormatter:
 
     def test_tool_calling_formatter_validation(self):
         """Test tool calling formatter validation."""
+
         formatter = ToolCallingFormatter()
 
         # Valid sample
-        valid_sample = {
-            "question": "Test question",
-            "tool_used": "test_tool",
-            "answer": "Test answer",
-        }
-        assert formatter.validate(valid_sample)
-
-        # Invalid samples
-        assert not formatter.validate({})  # Empty
-        assert not formatter.validate({"question": "Test"})  # Missing tool_used and answer
-        assert not formatter.validate({"tool_used": "test"})  # Missing question and answer
+        valid_sample = Conversation(
+            messages=[
+                ChatMessage(role="user", content="Test question"),
+                ChatMessage(role="assistant", content="Test answer"),
+            ],
+            reasoning=ReasoningTrace(style="freetext", content="Test analysis"),
+            tool_context=ToolContext(
+                available_tools=[],
+                executions=[
+                    ToolExecution(
+                        function_name="test_tool",
+                        arguments='{"param": "value"}',
+                        reasoning="Test reasoning",
+                        result="Test result",
+                    )
+                ],
+            ),
+            question="Test question",
+            final_answer="Test answer",
+        )
+        assert formatter.validate(valid_sample.model_dump())
 
     def test_tool_calling_formatter_custom_formats(self):
         """Test tool calling formatter with custom formatting."""
+
         config = {
             "thinking_format": "<!-- thinking: {reasoning} -->",
             "tool_call_format": "```json\n{tool_call}\n```",
@@ -761,22 +806,36 @@ class TestToolCallingFormatter:
         }
         formatter = ToolCallingFormatter(config)
 
-        sample = {
-            "question": "Test question",
-            "reasoning": "Test reasoning",
-            "tool_used": "test_tool",
-            "tool_input": "{}",
-            "tool_output": "test output",
-            "answer": "Test answer",
-        }
+        sample = Conversation(
+            messages=[
+                ChatMessage(role="user", content="Test question"),
+                ChatMessage(role="assistant", content="Test answer"),
+            ],
+            reasoning=ReasoningTrace(style="freetext", content="Test reasoning"),
+            tool_context=ToolContext(
+                available_tools=[],
+                executions=[
+                    ToolExecution(
+                        function_name="test_tool",
+                        arguments="{}",
+                        reasoning="Test tool reasoning",
+                        result="test output",
+                    )
+                ],
+            ),
+            question="Test question",
+            final_answer="Test answer",
+        )
 
         result = formatter.format_with_metadata([sample])
         formatted = result.samples[0]
         messages = formatted["messages"]
 
-        # Check custom thinking format
+        # Check custom thinking format markers are used
         assistant_msg = messages[1]["content"]
-        assert "<!-- thinking: Test reasoning -->" in assistant_msg
+        assert "<!-- thinking:" in assistant_msg
+        assert "-->" in assistant_msg
+        assert "Test reasoning" in assistant_msg
 
         # Check custom tool call format
         assert "```json" in assistant_msg
@@ -788,16 +847,30 @@ class TestToolCallingFormatter:
 
     def test_tool_calling_formatter_json_parsing(self):
         """Test tool calling formatter JSON parsing scenarios."""
+
         formatter = ToolCallingFormatter()
 
-        # Test with single quotes (should be converted)
-        sample = {
-            "question": "Test",
-            "tool_used": "test_tool",
-            "tool_input": "{'key': 'value'}",  # Single quotes
-            "tool_output": "output",
-            "answer": "answer",
-        }
+        # Test with single quotes in arguments string (will be parsed)
+        sample = Conversation(
+            messages=[
+                ChatMessage(role="user", content="Test"),
+                ChatMessage(role="assistant", content="answer"),
+            ],
+            reasoning=ReasoningTrace(style="freetext", content="Test analysis"),
+            tool_context=ToolContext(
+                available_tools=[],
+                executions=[
+                    ToolExecution(
+                        function_name="test_tool",
+                        arguments='{"key": "value"}',
+                        reasoning="Test reasoning",
+                        result="output",
+                    )
+                ],
+            ),
+            question="Test",
+            final_answer="answer",
+        )
 
         result = formatter.format_with_metadata([sample])
         formatted = result.samples[0]
@@ -1147,3 +1220,336 @@ class TestHarmonyFormatter:
 
         # Both should have commentary channel
         assert text.count("<|channel|>commentary") >= 2  # noqa: PLR2004
+
+
+class TestTRLSFTToolsFormatter:
+    """Test the TRL SFT Tools formatter."""
+
+    def setup_method(self):
+        """Set up test fixtures."""
+
+        self.formatter = TRLSFTToolsFormatter()
+
+    def test_formatter_initialization(self):
+        """Test formatter can be initialized."""
+
+        formatter = TRLSFTToolsFormatter()
+        assert formatter is not None
+
+    def test_formatter_with_config(self):
+        """Test formatter initialization with config."""
+
+        config = {
+            "include_system_prompt": True,
+            "validate_tool_schemas": True,
+            "remove_available_tools_field": False,
+        }
+        formatter = TRLSFTToolsFormatter(config)
+        assert formatter is not None
+
+    def test_validate_sample_with_messages(self):
+        """Test validation of sample with messages."""
+        sample = {
+            "messages": [
+                {"role": "user", "content": "Hello"},
+                {"role": "assistant", "content": "Hi!"},
+            ]
+        }
+        assert self.formatter.validate(sample)
+
+    def test_validate_sample_without_messages(self):
+        """Test validation fails without messages."""
+        sample = {"text": "Hello"}
+        assert not self.formatter.validate(sample)
+
+    def test_validate_sample_empty_messages(self):
+        """Test validation fails with empty messages."""
+        sample = {"messages": []}
+        assert not self.formatter.validate(sample)
+
+    def test_format_sample_with_available_tools(self):
+        """Test formatting sample with available_tools."""
+        sample = {
+            "messages": [
+                {"role": "user", "content": "What's the weather?"},
+                {"role": "assistant", "content": "Let me check..."},
+            ],
+            "available_tools": [
+                {
+                    "name": "get_weather",
+                    "description": "Get weather for a location",
+                    "parameters": [
+                        {
+                            "name": "location",
+                            "type": "str",
+                            "description": "City name",
+                            "required": True,
+                        }
+                    ],
+                    "returns": "Weather information",
+                    "category": "weather",
+                }
+            ],
+        }
+
+        result = self.formatter.format([sample])
+        assert len(result) == 1
+
+        # Convert FormattedOutput to dict
+        formatted = result[0].model_dump()
+        assert "messages" in formatted
+        assert "tools" in formatted
+        assert len(formatted["tools"]) == 1
+
+        # Check tool format
+        tool = formatted["tools"][0]
+        assert tool["type"] == "function"
+        assert "function" in tool
+        assert tool["function"]["name"] == "get_weather"
+        assert tool["function"]["description"] == "Get weather for a location"
+        assert "parameters" in tool["function"]
+
+    def test_format_sample_without_tools(self):
+        """Test formatting sample without available_tools."""
+        sample = {
+            "messages": [
+                {"role": "user", "content": "Hello"},
+                {"role": "assistant", "content": "Hi!"},
+            ]
+        }
+
+        result = self.formatter.format([sample])
+        assert len(result) == 1
+
+        # Convert FormattedOutput to dict
+        formatted = result[0].model_dump()
+        assert "messages" in formatted
+        # No tools field should be added if not present in original
+        assert "tools" not in formatted or formatted.get("tools") == []
+
+    def test_tool_schema_conversion(self):
+        """Test proper conversion of tools to OpenAI schema."""
+        sample = {
+            "messages": [{"role": "user", "content": "Test"}],
+            "available_tools": [
+                {
+                    "name": "calculator",
+                    "description": "Perform calculations",
+                    "parameters": [
+                        {
+                            "name": "expression",
+                            "type": "str",
+                            "description": "Math expression",
+                            "required": True,
+                        },
+                        {
+                            "name": "precision",
+                            "type": "int",
+                            "description": "Decimal precision",
+                            "required": False,
+                            "default": "2",
+                        },
+                    ],
+                    "returns": "Calculation result",
+                    "category": "math",
+                }
+            ],
+        }
+
+        result = self.formatter.format([sample])
+        formatted = result[0].model_dump()
+        tool = formatted["tools"][0]
+
+        # Verify OpenAI schema structure
+        assert tool["type"] == "function"
+        assert tool["function"]["name"] == "calculator"
+        assert tool["function"]["description"] == "Perform calculations"
+
+        params = tool["function"]["parameters"]
+        assert params["type"] == "object"
+        assert "properties" in params
+        assert "required" in params
+
+        # Check parameter conversion
+        assert "expression" in params["properties"]
+        assert params["properties"]["expression"]["type"] == "string"
+        assert "expression" in params["required"]
+
+        assert "precision" in params["properties"]
+        assert params["properties"]["precision"]["type"] == "integer"
+        assert params["properties"]["precision"]["default"] == 2  # noqa: PLR2004
+        assert "precision" not in params["required"]
+
+    def test_multiple_tools_conversion(self):
+        """Test conversion of multiple tools."""
+        sample = {
+            "messages": [{"role": "user", "content": "Test"}],
+            "available_tools": [
+                {
+                    "name": "tool1",
+                    "description": "First tool",
+                    "parameters": [],
+                    "returns": "Result",
+                    "category": "general",
+                },
+                {
+                    "name": "tool2",
+                    "description": "Second tool",
+                    "parameters": [],
+                    "returns": "Result",
+                    "category": "general",
+                },
+            ],
+        }
+
+        result = self.formatter.format([sample])
+        formatted = result[0].model_dump()
+        assert len(formatted["tools"]) == 2  # noqa: PLR2004
+        assert formatted["tools"][0]["function"]["name"] == "tool1"
+        assert formatted["tools"][1]["function"]["name"] == "tool2"
+
+    def test_remove_available_tools_field(self):
+        """Test removing available_tools field from output."""
+
+        config = {"remove_available_tools_field": True}
+        formatter = TRLSFTToolsFormatter(config)
+
+        sample = {
+            "messages": [{"role": "user", "content": "Test"}],
+            "available_tools": [
+                {
+                    "name": "test_tool",
+                    "description": "Test",
+                    "parameters": [],
+                    "returns": "Result",
+                    "category": "general",
+                }
+            ],
+        }
+
+        result = formatter.format([sample])
+        formatted = result[0].model_dump()
+
+        assert "tools" in formatted
+        assert "available_tools" not in formatted
+
+    def test_system_prompt_override(self):
+        """Test overriding system prompt."""
+
+        custom_prompt = "Custom system prompt for testing"
+        config = {
+            "include_system_prompt": True,
+            "system_prompt_override": custom_prompt,
+        }
+        formatter = TRLSFTToolsFormatter(config)
+
+        sample = {
+            "messages": [
+                {"role": "system", "content": "Original prompt"},
+                {"role": "user", "content": "Test"},
+            ]
+        }
+
+        result = formatter.format([sample])
+        formatted = result[0].model_dump()
+
+        assert formatted["messages"][0]["role"] == "system"
+        assert formatted["messages"][0]["content"] == custom_prompt
+
+    def test_add_system_prompt_if_missing(self):
+        """Test adding system prompt when missing."""
+
+        custom_prompt = "New system prompt"
+        config = {
+            "include_system_prompt": True,
+            "system_prompt_override": custom_prompt,
+        }
+        formatter = TRLSFTToolsFormatter(config)
+
+        sample = {"messages": [{"role": "user", "content": "Test"}]}
+
+        result = formatter.format([sample])
+        formatted = result[0].model_dump()
+
+        assert formatted["messages"][0]["role"] == "system"
+        assert formatted["messages"][0]["content"] == custom_prompt
+        assert len(formatted["messages"]) == 2  # noqa: PLR2004
+
+    def test_type_mapping(self):
+        """Test correct type mapping from DeepFabric to JSON Schema."""
+        sample = {
+            "messages": [{"role": "user", "content": "Test"}],
+            "available_tools": [
+                {
+                    "name": "test_types",
+                    "description": "Test type mapping",
+                    "parameters": [
+                        {
+                            "name": "str_param",
+                            "type": "str",
+                            "description": "String",
+                            "required": True,
+                        },
+                        {
+                            "name": "int_param",
+                            "type": "int",
+                            "description": "Integer",
+                            "required": True,
+                        },
+                        {
+                            "name": "float_param",
+                            "type": "float",
+                            "description": "Float",
+                            "required": True,
+                        },
+                        {
+                            "name": "bool_param",
+                            "type": "bool",
+                            "description": "Boolean",
+                            "required": True,
+                        },
+                        {
+                            "name": "list_param",
+                            "type": "list",
+                            "description": "List",
+                            "required": True,
+                        },
+                        {
+                            "name": "dict_param",
+                            "type": "dict",
+                            "description": "Dict",
+                            "required": True,
+                        },
+                    ],
+                    "returns": "Result",
+                    "category": "test",
+                }
+            ],
+        }
+
+        result = self.formatter.format([sample])
+        formatted = result[0].model_dump()
+        props = formatted["tools"][0]["function"]["parameters"]["properties"]
+
+        assert props["str_param"]["type"] == "string"
+        assert props["int_param"]["type"] == "integer"
+        assert props["float_param"]["type"] == "number"
+        assert props["bool_param"]["type"] == "boolean"
+        assert props["list_param"]["type"] == "array"
+        assert props["dict_param"]["type"] == "object"
+
+    def test_load_via_registry(self):
+        """Test loading TRL formatter via registry."""
+        registry = FormatterRegistry()
+        formatter = registry.load_formatter("builtin://trl_sft_tools")
+        assert formatter is not None
+
+        assert isinstance(formatter, TRLSFTToolsFormatter)
+
+    def test_example_config(self):
+        """Test getting example configuration."""
+        config = self.formatter.get_example_config()
+        assert "include_system_prompt" in config
+        assert "system_prompt_override" in config
+        assert "validate_tool_schemas" in config
+        assert "remove_available_tools_field" in config
